###########
#' cao()
#' The cao() function calculates the required amount of CaO (default in t/ha) using the soil pH, the soil organic matter (SOM, or soil organic carbon, SOC) content and the mean particle diameter (MDP) of the clay, silt and sand content. More under details.
#' @param TEX = (mandatory) if the MPD is available in the provided data set, use "MPD". If soil texture fractions clay, silt and sand are available, use "FRC".
#' @param SOC = (optional) logical. If set TRUE, data contains the soil organic carbon (SOC) content. If set FALSE, data set already contains SOM/humus content values. See details below.
#' @param unit = (optional) desired unit of the calculated CaO amounts. Default "t_ha"(t/ha). Also possible: "dt_ha"(dt/ha), "kg_ha" (dt/ha).
#' @param split = (optional) logical. Setting whether the total CaO demand should be split into up to three partial application amounts (Default = FALSE).
#' @param language = (optional) set "GER" to provide the output file in German language. Default is English ("ENG").
#' @keywords CaO-application, precision farming, pH_BB Project (http://ph-bb.com)
#' @details
#' 
#' This script is based on the work of Jörg Rühlmann; Eric Bönecke; Meyer, S. Predicting the lime demand of arable soils from pH value, soil texture and soil organic matter content. Agronomy 2021, 10, x. https://doi.org/10.3390/xxxxx. 
#' 
#' For TEX = "FRC", the input data.frame should contain the pH value, the soil organic carbon content (SOC) [M%], and the soil fractions clay, silt and sand [M%] as followed: 
#' 
#' data.frame(
#' pH = c(pH_value_1, pH_value_2, ..., pH_value_n), 
#' SOC = c(SOM_value_1, SOM_value_2, ..., SOM_value_n), 
#' CLAY = = c(clay_value_1, clay_value_2, ..., clay_value_n),
#' SILT = c(silt_value_1, silt_value_2, ..., silt_value_n), 
#' SAND = c(sand_value_1, sand_value_2, ..., sand_value_n)
#' )
#' 
#' or for TEX = "MPD":
#'  
#' data.frame(
#' pH = c(pH_value_1, pH_value_2, ..., pH_value_n), 
#' SOC = c(SOM_value_1, SOM_value_2, ..., SOM_value_n), 
#' MPD = c(MPD_value_1, MPD_value_2, ..., MPD_value_n)
#' )
#'
#' Note: 
#' - Upper cases for the header of the soil texture fractions are mandatory within the current version.
#' - Clay, silt and sand content as well as the SOM (or SOC) content should be provided in M%. 
#' - If SOC = TRUE: values will be multiplied by 1.724 to estimate the amount of soil organic matter (SOM, humus).
#' 
#' When the total amount of CaO calculated exceeds a certain threshold for a maximum recommended applicable CaO amount, the lime demand can be optionally split into two or three applications (split = TRUE), which should differ in their time of application. In accordance with the VDLUFA recommendations, the threshold is soil texture related and herein calculated based on the MPD by Equation 9 in the manuscript.
#' 
#' The function returns a data frame with the input parameters, the CaO application amount (either as a single column ["CaO"] = total CaO amount or if split is used four columns ["CaO"] = total CaO amount, ["CaO_T1"] = CaO amount for the first application, ["CaO_T2"]= CaO amount for the second application, ["CaO_T3"]= CaO amount for the third application), the pH class corresponding to the input pH-value following the VDALUFA guidelines, and the SOM class corresponding to the input SOM-value following the VDALUFA guidelines.
#' 
#' @examples 
#' # data set with soil fractions
#' df <- data.frame(
#'   pH = round(rnorm(100, 5, 1),1),
#'   SOC = round(rnorm(100, 4, 1),1)
#' )
#' mx <- matrix(rbeta(100*3,2,2), ncol=3)
#' tx <- as.data.frame(round(sweep(mx, 1, rowSums(mx), FUN="/")*100,2))
#' colnames(tx) <- c("CLAY", "SILT", "SAND")
#' df <- cbind(df, tx)
#' 
#' # data set with mean particle diameter
#' df <- data.frame(
#'   pH = round(rnorm(100, 5, 1),1),
#'   SOC = round(rnorm(100, 4, 1),1),
#' MPD = round(c(rnorm(50, .01, .0025), rnorm(50, .1, .025)),4)
#' )
#' CaO.df <- cao(x = df, TEX = "FRC", unit = "kg_ha", split = TRUE, language = "ENG")
#' 
cao <- function(x, TEX = NULL, SOC = NULL, unit = NULL, split = NULL, language = NULL) {
  
  # required packages
  require(soiltexture)
  
  #### defaults ####
  tex <- "FRC"
  soc <- TRUE
  unt <- "t/ha"
  spl <- FALSE
  lng <- "ENG"
  
  #### update input variables ####
  if (!is.null(TEX)) tex <- TEX
  if (!is.null(SOC)) soc <- SOC
  if (!is.null(unit)) unt <- unit
  if (!is.null(split)) spl <- split
  if (!is.null(language)) lng <- language
  
  #### prepare data frame ####
  dfp <- x # x <- df
  dfp$id <- 1:nrow(dfp)
  
  if (tex == "FRC") {
    
    if (is.null(dfp$pH)) {print("coloumn 'pH' does not exist")}
    if (is.null(dfp$SOC)) {print("coloumn 'SOC' does not exist")}
    if (is.null(dfp$CLAY)) {print("coloumn 'CLAY' does not exist")}
    if (is.null(dfp$SILT)) {print("coloumn 'SILT' does not exist")}
    if (is.null(dfp$SAND)) {print("coloumn 'SAND' does not exist")}
    
    # extract columns used for the algorithm
    df <- dfp[c("id", "pH", "SOC","CLAY","SILT","SAND")]
    df$pH <- round(df$pH,1)
    df$SOC <- round(df$SOC,1)
    df$CLAY <- round(df$CLAY,2)
    df$SILT <- round(df$SILT,2)
    df$SAND <- round(df$SAND,2)
    
    # extract columns not used for the algorithm (will be attached back later to the data frame)
    dt <- dfp[, -c(which(colnames(dfp) == "pH"),which(colnames(dfp) == "SOC"),which(colnames(dfp) == "CLAY"),which(colnames(dfp) == "SILT"),which(colnames(dfp) == "SAND"),which(colnames(dfp) == "id"))]

  } else {
    
    if (is.null(dfp$pH)) {print("coloumn 'pH' does not exist")}
    if (is.null(dfp$SOC)) {print("coloumn 'SOC' does not exist")}
    if (is.null(dfp$MPD)) {print("coloumn 'MPD' does not exist")}
    
    # extract columns used for the algorithm 
    df <- dfp[c("id", "pH", "SOC","MPD")]
    df$pH <- round(df$pH,1)
    df$SOC <- round(df$SOC,1)
    df$MPD <- round(df$MPD,4)
    
    # extract columns not used for the algorithm (will be later attached back to the data frame)
    dt <- dfp[, -c(which(colnames(dfp) == "pH"),which(colnames(dfp) == "SOC"),which(colnames(dfp) == "MPD"),which(colnames(dfp) == "id"))]
    
  }
  
  #### if not available, calculate mean particle diameter (MPD) based on the clay, silt and sand content (tex = "MPD") ####
  
  if (tex == "FRC") {
   
  # particle size boundaries [mm] according to the German Soil Science Society (DBG)
  cl.lo <- 0.00001 # lower clay boundary according to Shirazi et al. 1988
  cl.si <- 0.002   # clay-silt boundary
  si.sa <- 0.063   # silt-sand boundary
  sa.up <- 2       # upper sand boundary
  
  # geometric means of the three particle size fractions clay, silt and sand as per German Soil Science Society (DBG)
  # note: the log() transformation refers to the natural logarithm. Within Ruehlmann et al. 2020 it is describes as Ln() function.
  GM.cl <- exp(mean(log(c(cl.lo, cl.si)))) 
  GM.si <- exp(mean(log(c(cl.si, si.sa))))
  GM.sa <- exp(mean(log(c(si.sa, sa.up))))
  
  # geometric mean of each particle size fraction from the sampling data of the user
  FM.cl <- df$CLAY/100*log(GM.cl)
  FM.si <- df$SILT/100*log(GM.si)
  FM.sa <- df$SAND/100*log(GM.sa)
  
  # mean particle diameter (MPD) in 10^-3 m derived from the clay, silt and sand fraction of each sampling data -  Eq. 1
  df$MPD <- round(exp((FM.cl + FM.si + FM.sa)), 4)
   
  }
  
  #### calculate humus content in case input data only contains SOC values (SOC = TRUE) ####
  
  if (soc == TRUE) {
    df$SOM <- round(df$SOC * 1.724, 1)
    
    if (lng == "ENG") print(paste0("SOC content multiplied by 1.724 to get SOM content")) 
    if (lng == "GER") print(paste0("Corg-Gehalt mit 1.724 zur Berechnung des Humus-Gehalts multipliziert"))
    
  } else {
    df$SOM <- df$SOC
    df$SOC <- round(df$SOM / 1.724, 1)
  }
  
  #### calculate pH level thresholds, CaO amounts and allocate VDLUFA lime supply levels and SOM classes ####
  
  # include columns for the total CaO amounts
  df$CaO <- NA
  
  # include columns for CaO splits
  if (spl == TRUE) {
    
    if (lng == "ENG") print(paste0("CaO amount split into several application parts"))
    if (lng == "GER") print(paste0("CaO-Mengen in Teilgaben unterteilt"))
    
    df$CaO_t1 <- NA  # for the first application amount
    df$CaO_t2 <- NA  # for the second application amount
    df$CaO_t3 <- NA  # for the third application amount
  }
  
  txt.ndf = txtProgressBar(min = 0, max = nrow(df), initial = 0) 
  
  if (unt == "kg_ha") ut <- "kg/ha"
  if (unt == "dt_ha") ut <- "dt/ha"
  if (unt == "t_ha") ut <- "t/ha"
  
  if (lng == "ENG") print(paste0("Calculating CaO demands in ", ut))
  if (lng == "GER") print(paste0("CaO Mengenbedarfsberechnung in ", ut))
  
  # CAO estimation routine in Mg/ha (t/ha)
  
  for (q in 1:nrow(df)) {
    
    #### initial clay and humus values ####
    
    x <- df$MPD[q]
    y <- df$SOM[q]
    z <- df$pH[q]
    
    #### progress report ####
    
    # progress bar
    setTxtProgressBar(txt.ndf,q) 
    
    # progress status
    for (qq in 1:10) {
      
      if (q == nrow(df)*.1*qq) {
        
        if (lng == "GER") print(paste0(round(q / nrow(df) * 100), "% berechnet"))
        if (lng == "ENG") print(paste0(round(q / nrow(df) * 100), "% completed"))
        
      }
      
    }
    
    #### calculate lime supply level thresholds based on the mean particle diameter and the humus content ####
    
    #### upper pH limit for lime supply level A - Eq.3 in Ruehlmann et al. (2020) ####
    
    pHmax_A = 4.13 - 0.237*log(x) - 0.00517*y + 0.0734*log(x)^2 + 5.31e-06*y^2 + 0.00172*y*log(x) + 0.0134*log(x)^3 + 3e-08*y^3 - 9.94e-08*y^2*log(x) + 0.000178*y*log(x)^2
    
    #### upper pH limit for lime supply level B - Eq.4 in Ruehlmann et al. (2020) ####
    
    pHmax_B = 6.69 - 11.7*x - 0.0181*y + 63.1*x^2 + 0.0000794*y^2 + 0.0174*x*y - 163*x^3 - 1.39e-07*y^3 - 0.000056*x*y^2 + 0.00768*x^2*y
    
    #### upper pH limit for lime supply level C - Eq.5 in Ruehlmann et al. (2020) ####
    
    pHmax_C = (7.64 + 10.8*x - 10.3*x^2 - 0.0304*y + 0.000105*y^2 - 1.95e-07*y^3) / (1 + 4.28*x - 14.9*x^2 + 36.2*x^3 - 0.00155*y)
    
    #### upper pH limit for lime supply level D - Eq.6 in Ruehlmann et al. (2020) ####
    
    pHmax_D = 4.87 - 1.15*log(x) - 0.00826*y - 0.133*log(x)^2 + 9.51e-06*y^2 + 0.00146*y*log(x) - 0.00479*log(x)^3 + 3.54e-08*y^3 - 1.83e-06*y^2*log(x) + 0.0000913*y*log(x)^2
    
    #### print upper pH limits A - D ####
    
    pHmax_A
    pHmax_B
    pHmax_C
    pHmax_D
    
    #### intercept (int) and slope (slp) based on the relationship between the MPD and the SOM content ####
    
    # intercept as in Eq.7 in Ruehlmann et al. (2020) for the simple linear function (Eq.2)
    int = (-1.32 - 8.64*log(x) + 0.145*y + 0.0371*log(x)^2 - 0.000518*y^2 + 0.0364*y*log(x)) / (1 + 0.101*log(x) + 0.000439*y + 0.017*log(x)^2 - 7.33e-07*y^2 + 0.000222*y*log(x))
    
    # slope as in Eq.7 in Ruehlmann et al. (2020) for the simple linear function (Eq.2)
    slp = (0.469 - 1.12*log(x) + 0.0224*y + 0.0128*log(x)^2 - 8.45e-05*y^2 + 0.00502*y*log(x)) / (1 + 0.082*log(x) - 0.00175*y + 0.0144*log(x)^2 + 7.71e-06*y^2 + 0.000208*y*log(x))
    
    int
    slp
    
    #### CaO amounts for supply level A and B ####
    
    if (z <= pHmax_B) {
      
      df$CaO[q] <- int - slp*z # Eq.2 in Ruehlmann et al. (2020)
      
    }
    
    #### CaO amounts for supply level C ####
    
    # linear decrease of the applied CaO amount 
    # for the pH range of the supply level C
    #
    #  CaO (dt/ha)
    #   ^
    # 20|
    #   |-------- Standard amount for level C (e.g 17)
    # 15|\
    #   | \
    # 10|  \  
    #   |   \ 
    #  5|    \ 
    #   |     \ 
    #  0|______\_> pH
    #   5.8   6.3
    
    if (z > pHmax_B & z <= pHmax_C) {
      
      # CaO minimum for the upper pH boundary of the lime supply level B
      CaOmin_B <- int - slp*pHmax_B # Eq.2 in Ruehlmann et al. (2020)
      
      # Linear decreasing function from the upper pH boundary of the lime supply level B to the upper pH boundary of the lime supply level C
      df$CaO[q] = CaOmin_B * (pHmax_C - z) / (pHmax_C - pHmax_B) # Eq.10 in Ruehlmann et al. (2020)
      
    } 
    
    #### CaO amounts for supply level D and E ####
    
    if (z > pHmax_C) df$CaO[q] <- 0
    
    #### check for maximum CaO application amounts based on the soil texture and split amounts, if necessary ####
    
    # actual maximum CaO amounts for each soil texture (using the MPD as base)
    CaOmax = ((8.7 - 2.7) * (exp(-16.8*x)) + 2.7)  # Eq.9 in Ruehlmann et al. (2020)
    
    #### split CaO amounts in different application amounts ####
    
    if (spl == TRUE) {
      
      # first split, if CaO amounts are higher than recommended by the VDLUFA for a soil texture
      if (df$CaO[q] > CaOmax) {
        
        df$CaO_t1[q] <- df$CaO[q] - (df$CaO[q] - CaOmax)
        df$CaO_t2[q] <- df$CaO[q] - CaOmax
        
      } else {
        
        df$CaO_t1[q] <- df$CaO[q] 
        df$CaO_t2[q] <- 0
        
      }
      
      # second split, if CaO amounts are still higher within the second application amount than recommended by the VDLUFA for a soil texture
      if (df$CaO_t2[q] > CaOmax) {
        
        df$CaO_t3[q] <- df$CaO_t2[q] - CaOmax
        df$CaO_t2[q] <- df$CaO_t2[q] - (df$CaO_t2[q] - CaOmax)
        
      } else {
        
        df$CaO_t2[q] <- df$CaO_t2[q] 
        df$CaO_t3[q] <- 0
        
      }
      
    }
    
    #### allocate lime supply levels ####
    
    if (z <= pHmax_A) df$pH_class[q] <- "A"
    
    if (z > pHmax_A & z <= pHmax_B) df$pH_class[q] <- "B" 
    
    if (z > pHmax_B & z <= pHmax_C) df$pH_class[q] <- "C" 
    
    if (z > pHmax_C & z <= pHmax_D) df$pH_class[q] <- "D"
    
    if (z > pHmax_D) df$pH_class[q] <- "E"
    
    #### allocate SOM class ####
    
    if (df$SOM[q] <= 4) df$SOM_class[q] <- 1
    
    if (df$SOM[q] > 4 & df$SOM[q] <= 8) df$SOM_class[q] <- 2
    
    if (df$SOM[q] > 8 & df$SOM[q] <= 15) df$SOM_class[q] <- 3 
    
    if (df$SOM[q] > 15) df$SOM_class[q] <- 4
    
    #####
    
  }
  
  #### unit transformation and rounding ####
  
  if (unt == "kg_ha") {
    
    df$CaO <- round(df$CaO*1000, 1)
    
    if (spl == TRUE) {
      df$CaO_t1 <- round(df$CaO_t1*1000, 1)
      df$CaO_t2 <- round(df$CaO_t2*1000, 1)
      df$CaO_t3 <- round(df$CaO_t3*1000, 1)
    }
    
  }
  
  if (unt == "dt_ha") {
    
    df$CaO <- round(df$CaO*100, 1)
    
    if (spl == TRUE) {
      df$CaO_t1 <- round(df$CaO_t1*100, 1)
      df$CaO_t2 <- round(df$CaO_t2*100, 1)
      df$CaO_t3 <- round(df$CaO_t3*100, 1)
    }
    
  }
  
  if (unt == "t_ha") {
    
    df$CaO <- round(df$CaO, 3)
    
    if (spl == TRUE) {
      df$CaO_t1 <- round(df$CaO_t1, 3)
      df$CaO_t2 <- round(df$CaO_t2, 3)
      df$CaO_t3 <- round(df$CaO_t3, 3)
    }
    
  }
  
  #### rename and sort data ####
  
  if (lng == "GER") {
    
    if (spl == TRUE) {
      
      if (tex == "FRC") {
        
        colnames(df) <- c("ID", "pH", "Corg", "Ton", "Schluff", "Sand", "MPD", "Humus", "CaO", "CaO_T1", "CaO_T2", "CaO_T3", "pH_Klasse", "Humus_Klasse")
        
        #df <- df[,c("ID", "pH", "pH_Klasse", "Corg", "Humus", "Humus_Klasse", "Ton", "Schluff", "Sand", "MPD", "CaO", "CaO_T1", "CaO_T2", "CaO_T3")]
        
      } else {
        
        colnames(df) <- c("ID", "pH", "Corg", "MPD", "Humus", "CaO", "CaO_T1", "CaO_T2", "CaO_T3", "pH_Klasse", "Humus_Klasse")
        
        #df <- df[,c("ID", "pH", "pH_Klasse", "Corg", "Humus", "Humus_Klasse", "MPD", "CaO", "CaO_T1", "CaO_T2", "CaO_T3")]
        
      }
      
    } else {
      
      if (tex == "FRC") {
        
        colnames(df) <- c("ID", "pH", "Corg", "Ton", "Schluff", "Sand", "MPD", "Humus", "CaO", "pH_Klasse", "Humus_Klasse")
        
        #df <- df[,c("ID", "pH", "pH_Klasse", "Corg", "Humus", "Humus_Klasse", "Ton", "Schluff", "Sand", "MPD", "CaO")]
        
      } else {
        
        colnames(df) <- c("ID", "pH", "Corg", "MPD", "Humus", "CaO", "pH_Klasse", "Humus_Klasse")
        
        #df <- df[,c("ID", "pH", "pH_Klasse", "Corg", "Humus", "Humus_Klasse", "MPD", "CaO")]
        
      }
      
    }
    
  } else {
    
    if (spl == TRUE) {
      
      if (tex == "FRC") {
        
        colnames(df) <- c("ID", "pH", "SOC", "Clay", "Silt", "Sand", "MPD", "SOM", "CaO", "CaO_t1", "CaO_t2", "CaO_t3", "pH_level", "SOM_class")
        
        #df <- df[,c("ID", "pH", "pH_level", "SOC", "SOM", "SOM_class", "Clay", "Silt", "Sand", "MPD", "CaO", "CaO_t1", "CaO_t2", "CaO_t3")]
        
      } else {
        
        colnames(df) <- c("ID", "pH", "SOC", "MPD", "SOM", "CaO", "CaO_T1", "CaO_T2", "CaO_T3", "pH_level", "SOM_class")
        
        #df <- df[,c("ID", "pH", "pH_level", "SOC", "SOM", "SOM_class", "MPD", "CaO", "CaO_T1", "CaO_T2", "CaO_T3")]
        
      }
      
    } else {
      
      if (tex == "FRC") {
        
        colnames(df) <- c("ID", "pH", "SOC", "Clay", "Silt", "Sand", "MPD", "SOM", "CaO", "pH_level", "SOM_class")
        
        #df <- df[,c("ID", "pH", "pH_level", "SOC", "SOM", "SOM_class", "Clay", "Silt", "Sand", "MPD", "CaO")]
        
      } else {
        
        colnames(df) <- c("ID", "pH", "SOC", "MPD", "SOM", "CaO", "pH_level", "SOM_class")
        
        #df <- df[,c("ID", "pH", "pH_level", "SOC", "SOM", "SOM_class", "MPD", "CaO")]
        
      }
      
    }
    
  }
  
  df <- cbind(df, dt)
  
}
